@@ -20,20 +20,28 @@ from typing import Callable, Any
 
 from jax import core
 from jax import tree_util
+from jax import lax
+from jax._src import util
 from jax.interpreters import ad
 from jax.interpreters import batching
 from jax.interpreters import mlir
+from jax._src.lax import control_flow as lcf
+import jax.numpy as jnp
 
 DebugEffect = enum.Enum('DebugEffect', ['PRINT', 'ORDERED_PRINT'])
 
 core.ordered_effects.add(DebugEffect.ORDERED_PRINT)
 mlir.lowerable_effects.add(DebugEffect.PRINT)
 mlir.lowerable_effects.add(DebugEffect.ORDERED_PRINT)
+lcf.allowed_effects.add(DebugEffect.PRINT)
+lcf.allowed_effects.add(DebugEffect.ORDERED_PRINT)
 
 # `debug_callback_p` is the main primitive for staging out Python callbacks.
 debug_callback_p = core.Primitive('debug_callback')
 debug_callback_p.multiple_results = True
 
+map, unsafe_map = util.safe_map, map
+
 @debug_callback_p.def_impl
 def debug_callback_impl(*flat_args, callback: Callable[..., Any],
     effect: DebugEffect, in_tree: tree_util.PyTreeDef):
@@ -48,25 +56,38 @@ def debug_callback_abstract_eval(*flat_avals, callback: Callable[..., Any],
   del flat_avals, callback, in_tree
   return [], {effect}
 
-def debug_callback_batching_rule(*flat_args, callback: Callable[..., Any],
-    effect: DebugEffect, in_tree: tree_util.PyTreeDef):
-  del flat_args, callback, effect, in_tree
-  # TODO(sharadmv): implement batching rule
-  raise NotImplementedError('Batching not supported for `debug_callback`.')
+def debug_callback_batching_rule(args, dims, **params):
+  """Unrolls the debug callback across the mapped axis."""
+  axis_size = next(x.shape[i] for x, i in zip(args, dims)
+                   if i is not None)
+  # TODO(sharadmv): implement in terms of rolled loop unstead of
+  # unrolled.
+  def get_arg_at_dim(i, dim, arg):
+    if dim is batching.not_mapped:
+      # Broadcast unmapped argument
+      return arg
+    return lax.index_in_dim(arg, i, axis=dim, keepdims=False)
+  outs = []
+  for i in range(axis_size):
+    args_idx = map(functools.partial(get_arg_at_dim, i), dims, args)
+    outs.append(debug_callback_p.bind(*args_idx, **params))
+  outs = [jnp.stack(xs) for xs in zip(*outs)]
+  return outs, (0,) * len(outs)
 batching.primitive_batchers[debug_callback_p] = debug_callback_batching_rule
 
 def debug_callback_jvp_rule(*flat_args, callback: Callable[..., Any],
     effect: DebugEffect, in_tree: tree_util.PyTreeDef):
   del flat_args, callback, effect, in_tree
-  # TODO(sharadmv): implement jvp rule
-  raise NotImplementedError('JVP not supported for `debug_callback`.')
+  # TODO(sharadmv): link to relevant documentation when it exists
+  raise ValueError(
+      "JVP doesn't support debugging callbacks. "
+      "Instead, you can use them with `jax.custom_jvp` or `jax.custom_vjp`.")
 ad.primitive_jvps[debug_callback_p] = debug_callback_jvp_rule
 
 def debug_callback_transpose_rule(*flat_args, callback: Callable[..., Any],
     effect: DebugEffect, in_tree: tree_util.PyTreeDef):
   del flat_args, callback, effect, in_tree
-  # TODO(sharadmv): implement transpose rule
-  raise NotImplementedError('Transpose not supported for `debug_callback`.')
+  raise ValueError("Transpose doesn't support debugging callbacks.")
 ad.primitive_transposes[debug_callback_p] = debug_callback_transpose_rule
 
 def _ordered_effect_lowering(ctx, token, *args, **params):
