@@ -13,11 +13,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import inspect
 import functools
 from functools import partial
+import warnings
 
 import numpy as np
 
+import jax
 from jax._src.numpy import lax_numpy as jnp
 from jax._src.numpy.vectorize import vectorize
 from jax._src import ad_util
@@ -37,6 +40,10 @@ from jax._src.lax import lax as lax_internal
 from jax._src.lax import svd as lax_svd
 from jax._src.lib import lapack
 
+from jax._src.lib import gpu_linalg
+from jax._src.lib import gpu_solver
+from jax._src.lib import gpu_sparse
+
 from jax._src.lib import cuda_linalg
 from jax._src.lib import hip_linalg
 from jax._src.lib import sparse_apis
@@ -53,7 +60,52 @@ xops = xla_client.ops
 
 # traceables
 
-def cholesky(x, symmetrize_input: bool = True):
+# TODO(phawkins): remove backward compatibility shim after 2022/08/11.
+def _warn_on_positional_kwargs(f):
+  """Decorator used for backward compatibility of keyword-only arguments.
+
+  Some functions were changed to mark their keyword arguments as keyword-only.
+  This decorator allows existing code to keep working temporarily, while issuing
+  a warning if a now keyword-only parameter is passed positionally."""
+  sig = inspect.signature(f)
+  pos_names = [name for name, p in sig.parameters.items()
+               if p.kind == inspect.Parameter.POSITIONAL_OR_KEYWORD]
+  kwarg_names = [name for name, p in sig.parameters.items()
+                 if p.kind == inspect.Parameter.KEYWORD_ONLY]
+
+  # This decorator assumes that all arguments to `f` are either
+  # positional-or-keyword or keyword-only.
+  assert len(pos_names) + len(kwarg_names) == len(sig.parameters)
+
+  @functools.wraps(f)
+  def wrapped(*args, **kwargs):
+    if len(args) < len(pos_names):
+      a = pos_names[len(args)]
+      raise TypeError(f"{f.__name__} missing required positional argument: {a}")
+
+    pos_args = args[:len(pos_names)]
+    extra_kwargs = args[len(pos_names):]
+
+    if len(extra_kwargs) > len(kwarg_names):
+      raise TypeError(f"{f.__name__} takes at most {len(sig.parameters)} "
+                      f" arguments but {len(args)} were given.")
+
+    for name, value in zip(kwarg_names, extra_kwargs):
+      if name in kwargs:
+        raise TypeError(f"{f.__name__} got multiple values for argument: "
+                        f"{name}")
+
+      warnings.warn(f"Argument {name} to {f.__name__} is now a keyword-only "
+                    "argument. Support for passing it positionally will be "
+                    "removed in an upcoming JAX release.",
+                    DeprecationWarning)
+      kwargs[name] = value
+    return f(*pos_args, **kwargs)
+
+  return wrapped
+
+@_warn_on_positional_kwargs
+def cholesky(x, *, symmetrize_input: bool = True):
   """Cholesky decomposition.
 
   Computes the Cholesky decomposition
@@ -82,7 +134,8 @@ def cholesky(x, symmetrize_input: bool = True):
     x = symmetrize(x)
   return jnp.tril(cholesky_p.bind(x))
 
-def eig(x, compute_left_eigenvectors=True, compute_right_eigenvectors=True):
+@_warn_on_positional_kwargs
+def eig(x, *, compute_left_eigenvectors=True, compute_right_eigenvectors=True):
   """Eigendecomposition of a general matrix.
 
   Nonsymmetric eigendecomposition is at present only implemented on CPU.
@@ -90,8 +143,10 @@ def eig(x, compute_left_eigenvectors=True, compute_right_eigenvectors=True):
   return eig_p.bind(x, compute_left_eigenvectors=compute_left_eigenvectors,
                     compute_right_eigenvectors=compute_right_eigenvectors)
 
-def eigh(x, lower: bool = True, symmetrize_input: bool = True):
-  """Eigendecomposition of a Hermitian matrix.
+@_warn_on_positional_kwargs
+def eigh(x, *, lower: bool = True, symmetrize_input: bool = True,
+         sort_eigenvalues: bool = True, ):
+  r"""Eigendecomposition of a Hermitian matrix.
 
   Computes the eigenvectors and eigenvalues of a complex Hermitian or real
   symmetric square matrix.
@@ -104,7 +159,10 @@ def eigh(x, lower: bool = True, symmetrize_input: bool = True):
       triangle given by ``lower`` is accessed; the other triangle is ignored and
       not accessed.
     symmetrize_input: If ``True``, the matrix is symmetrized before the
-      eigendecomposition by computing :math:`\\frac{1}{2}(x + x^H)`.
+      eigendecomposition by computing :math:`\frac{1}{2}(x + x^H)`.
+    sort_eigenvalues: If ``True``, the eigenvalues will be sorted in ascending
+      order. If ``False`` the eigenvalues are returned in an
+      implementation-defined order.
 
   Returns:
     A tuple ``(v, w)``.
@@ -118,7 +176,7 @@ def eigh(x, lower: bool = True, symmetrize_input: bool = True):
   """
   if symmetrize_input:
     x = symmetrize(x)
-  v, w = eigh_p.bind(x, lower=lower)
+  v, w = eigh_p.bind(x, lower=lower, sort_eigenvalues=sort_eigenvalues)
   return v, w
 
 
@@ -173,7 +231,8 @@ def lu(x):
   lu, pivots, permutation = lu_p.bind(x)
   return lu, pivots, permutation
 
-def qr(x, full_matrices: bool = True):
+@_warn_on_positional_kwargs
+def qr(x, *, full_matrices: bool = True):
   """QR decomposition.
 
   Computes the QR decomposition
@@ -204,7 +263,8 @@ def qr(x, full_matrices: bool = True):
   return q, r
 
 # TODO: Add `max_qdwh_iterations` to the function signature for TPU SVD.
-def svd(x, full_matrices=True, compute_uv=True):
+@_warn_on_positional_kwargs
+def svd(x, *, full_matrices=True, compute_uv=True):
   """Singular value decomposition.
 
   Returns the singular values if compute_uv is False, otherwise returns a triple
@@ -219,7 +279,8 @@ def svd(x, full_matrices=True, compute_uv=True):
     s, = result
     return s
 
-def triangular_solve(a, b, left_side: bool = False, lower: bool = False,
+@_warn_on_positional_kwargs
+def triangular_solve(a, b, *, left_side: bool = False, lower: bool = False,
                      transpose_a: bool = False, conjugate_a: bool = False,
                      unit_diagonal: bool = False):
   r"""Triangular solve.
@@ -321,7 +382,7 @@ _cpu_lapack_types = {np.dtype(np.float32), np.dtype(np.float64),
 
 # Cholesky decomposition
 
-def cholesky_jvp_rule(primals, tangents):
+def _cholesky_jvp_rule(primals, tangents):
   x, = primals
   sigma_dot, = tangents
   L = jnp.tril(cholesky_p.bind(x))
@@ -340,20 +401,23 @@ def cholesky_jvp_rule(primals, tangents):
       precision=lax.Precision.HIGHEST)
   return L, L_dot
 
-def cholesky_batching_rule(batched_args, batch_dims):
+def _cholesky_batching_rule(batched_args, batch_dims):
   x, = batched_args
   bd, = batch_dims
   x = batching.moveaxis(x, bd, 0)
   return cholesky(x), 0
 
 cholesky_p = standard_unop(_float | _complex, 'cholesky')
-ad.primitive_jvps[cholesky_p] = cholesky_jvp_rule
-batching.primitive_batchers[cholesky_p] = cholesky_batching_rule
+ad.primitive_jvps[cholesky_p] = _cholesky_jvp_rule
+batching.primitive_batchers[cholesky_p] = _cholesky_batching_rule
 
 def _cholesky_lowering(ctx, x):
-  aval, = ctx.avals_out
-  return mhlo.CholeskyOp(mlir.aval_to_ir_type(aval), x,
-                         lower=ir.BoolAttr.get(True)).results
+  if jax._src.lib.mlir_api_version < 18:
+    aval, = ctx.avals_out
+    return mhlo.CholeskyOp(mlir.aval_to_ir_type(aval), x,
+                           lower=ir.BoolAttr.get(True)).results
+  else:
+    return mhlo.CholeskyOp(x, lower=ir.BoolAttr.get(True)).results
 
 mlir.register_lowering(cholesky_p, _cholesky_lowering)
 
@@ -377,6 +441,16 @@ mlir.register_lowering(
     partial(_cholesky_cpu_gpu_lowering, lapack.potrf_mhlo),
     platform='cpu')
 
+if gpu_solver is not None:
+  mlir.register_lowering(
+    cholesky_p,
+    partial(_cholesky_cpu_gpu_lowering, gpu_solver.cuda_potrf),
+    platform='cuda')
+  mlir.register_lowering(
+    cholesky_p,
+    partial(_cholesky_cpu_gpu_lowering, gpu_solver.rocm_potrf),
+    platform='rocm')
+
 if solver_apis is not None:
   mlir.register_lowering(
     cholesky_p,
@@ -500,17 +574,19 @@ ad.primitive_jvps[eig_p] = eig_jvp_rule
 
 # Symmetric/Hermitian eigendecomposition
 
-def eigh_impl(operand, lower):
-  v, w = xla.apply_primitive(eigh_p, operand, lower=lower)
+def _eigh_impl(operand, *, lower, sort_eigenvalues):
+  v, w = xla.apply_primitive(eigh_p, operand, lower=lower,
+                             sort_eigenvalues=sort_eigenvalues)
   return v, w
 
-def _eigh_translation_rule(ctx, avals_in, avals_out, operand, *, lower):
+def _eigh_translation_rule(ctx, avals_in, avals_out, operand, *, lower,
+                           sort_eigenvalues):
   operand_aval, = avals_in
   if operand_aval.shape[-1] == 0:
     return [operand, xops.Real(xops.Reshape(operand, operand_aval.shape[:-1]))]
-  return xops.Eigh(operand, lower=lower)
+  return xops.Eigh(operand, lower=lower, sort_eigenvalues=sort_eigenvalues)
 
-def eigh_abstract_eval(operand, lower):
+def _eigh_abstract_eval(operand, *, lower, sort_eigenvalues):
   if isinstance(operand, ShapedArray):
     if operand.ndim < 2 or operand.shape[-2] != operand.shape[-1]:
       raise ValueError(
@@ -526,7 +602,9 @@ def eigh_abstract_eval(operand, lower):
     v, w = operand, operand
   return v, w
 
-def _eigh_cpu_gpu_lowering(syevd_impl, ctx, operand, *, lower):
+def _eigh_cpu_gpu_lowering(syevd_impl, ctx, operand, *, lower,
+                           sort_eigenvalues):
+  del sort_eigenvalues  # The CPU/GPU implementations always sort.
   operand_aval, = ctx.avals_in
   v_aval, w_aval = ctx.avals_out
   batch_dims = operand_aval.shape[:-2]
@@ -547,7 +625,7 @@ def _eigh_cpu_gpu_lowering(syevd_impl, ctx, operand, *, lower):
       w, _nan_like_mhlo(w_aval))
   return [v, w]
 
-def eigh_jvp_rule(primals, tangents, lower):
+def _eigh_jvp_rule(primals, tangents, *, lower, sort_eigenvalues):
   # Derivative for eigh in the simplest case of distinct eigenvalues.
   # This is classic nondegenerate perurbation theory, but also see
   # https://people.maths.ox.ac.uk/gilesm/files/NA-08-01.pdf
@@ -559,7 +637,8 @@ def eigh_jvp_rule(primals, tangents, lower):
   a, = primals
   a_dot, = tangents
 
-  v, w_real = eigh_p.bind(symmetrize(a), lower=lower)
+  v, w_real = eigh_p.bind(symmetrize(a), lower=lower,
+                          sort_eigenvalues=sort_eigenvalues)
 
   # for complex numbers we need eigenvalues to be full dtype of v, a:
   w = w_real.astype(a.dtype)
@@ -574,24 +653,33 @@ def eigh_jvp_rule(primals, tangents, lower):
   dw = jnp.real(jnp.diagonal(vdag_adot_v, axis1=-2, axis2=-1))
   return (v, w_real), (dv, dw)
 
-def eigh_batching_rule(batched_args, batch_dims, lower):
+def _eigh_batching_rule(batched_args, batch_dims, *, lower, sort_eigenvalues):
   x, = batched_args
   bd, = batch_dims
   x = batching.moveaxis(x, bd, 0)
-  return eigh_p.bind(x, lower=lower), (0, 0)
+  return eigh_p.bind(x, lower=lower, sort_eigenvalues=sort_eigenvalues), (0, 0)
 
 eigh_p = Primitive('eigh')
 eigh_p.multiple_results = True
-eigh_p.def_impl(eigh_impl)
-eigh_p.def_abstract_eval(eigh_abstract_eval)
+eigh_p.def_impl(_eigh_impl)
+eigh_p.def_abstract_eval(_eigh_abstract_eval)
 xla.register_translation(eigh_p, _eigh_translation_rule)
-ad.primitive_jvps[eigh_p] = eigh_jvp_rule
-batching.primitive_batchers[eigh_p] = eigh_batching_rule
+ad.primitive_jvps[eigh_p] = _eigh_jvp_rule
+batching.primitive_batchers[eigh_p] = _eigh_batching_rule
 
 mlir.register_lowering(
     eigh_p, partial(_eigh_cpu_gpu_lowering, lapack.syevd_mhlo),
     platform='cpu')
 
+if gpu_solver is not None:
+  mlir.register_lowering(
+    eigh_p, partial(_eigh_cpu_gpu_lowering, gpu_solver.cuda_syevd),
+    platform='cuda')
+  mlir.register_lowering(
+    eigh_p, partial(_eigh_cpu_gpu_lowering, gpu_solver.rocm_syevd),
+    platform='rocm')
+
+
 if solver_apis is not None:
   mlir.register_lowering(
     eigh_p, partial(_eigh_cpu_gpu_lowering, solver_apis.syevd_mhlo),
@@ -602,7 +690,7 @@ triangular_solve_dtype_rule = partial(
     naryop_dtype_rule, _input_dtype, (_float | _complex, _float | _complex),
     'triangular_solve')
 
-def triangular_solve_shape_rule(a, b, left_side=False, **unused_kwargs):
+def triangular_solve_shape_rule(a, b, *, left_side=False, **unused_kwargs):
   if a.ndim < 2:
     msg = "triangular_solve requires a.ndim to be at least 2, got {}."
     raise TypeError(msg.format(a.ndim))
@@ -624,7 +712,8 @@ def triangular_solve_shape_rule(a, b, left_side=False, **unused_kwargs):
   return b.shape
 
 def triangular_solve_jvp_rule_a(
-    g_a, ans, a, b, left_side, lower, transpose_a, conjugate_a, unit_diagonal):
+    g_a, ans, a, b, *, left_side, lower, transpose_a, conjugate_a,
+    unit_diagonal):
   m, n = b.shape[-2:]
   k = 1 if unit_diagonal else 0
   g_a = jnp.tril(g_a, k=-k) if lower else jnp.triu(g_a, k=k)
@@ -635,8 +724,9 @@ def triangular_solve_jvp_rule_a(
                 precision=lax.Precision.HIGHEST)
 
   def a_inverse(rhs):
-    return triangular_solve(a, rhs, left_side, lower, transpose_a, conjugate_a,
-                            unit_diagonal)
+    return triangular_solve(a, rhs, left_side=left_side, lower=lower,
+                            transpose_a=transpose_a, conjugate_a=conjugate_a,
+                            unit_diagonal=unit_diagonal)
 
   # triangular_solve is about the same cost as matrix multplication (~n^2 FLOPs
   # for matrix/vector inputs). Order these operations in whichever order is
@@ -655,7 +745,7 @@ def triangular_solve_jvp_rule_a(
       return dot(ans, a_inverse(g_a))  # X (∂A A^{-1})
 
 def triangular_solve_transpose_rule(
-    cotangent, a, b, left_side, lower, transpose_a, conjugate_a,
+    cotangent, a, b, *, left_side, lower, transpose_a, conjugate_a,
     unit_diagonal):
   # Triangular solve is nonlinear in its first argument and linear in its second
   # argument, analogous to `div` but swapped.
@@ -663,12 +753,14 @@ def triangular_solve_transpose_rule(
   if type(cotangent) is ad_util.Zero:
     cotangent_b = ad_util.Zero(b.aval)
   else:
-    cotangent_b = triangular_solve(a, cotangent, left_side, lower,
-                                   not transpose_a, conjugate_a, unit_diagonal)
+    cotangent_b = triangular_solve(a, cotangent, left_side=left_side,
+                                   lower=lower, transpose_a=not transpose_a,
+                                   conjugate_a=conjugate_a,
+                                   unit_diagonal=unit_diagonal)
   return [None, cotangent_b]
 
 
-def triangular_solve_batching_rule(batched_args, batch_dims, left_side,
+def triangular_solve_batching_rule(batched_args, batch_dims, *, left_side,
                                    lower, transpose_a, conjugate_a,
                                    unit_diagonal):
   x, y = batched_args
@@ -775,6 +867,16 @@ def _triangular_solve_gpu_lower(
                                   ir.BoolAttr.get(unit_diagonal),
                                   mhlo.TransposeAttr.get(transpose)).results
 
+if gpu_solver is not None:
+  mlir.register_lowering(
+      triangular_solve_p,
+      partial(_triangular_solve_gpu_lower, gpu_solver.cuda_trsm),
+      platform='cuda')
+  mlir.register_lowering(
+      triangular_solve_p,
+      partial(_triangular_solve_gpu_lower, gpu_solver.rocm_trsm),
+      platform='rocm')
+
 if solver_apis is not None:
   mlir.register_lowering(
       triangular_solve_p,
@@ -853,13 +955,9 @@ def _lu_pivots_to_permutation_batching_rule(batched_args, batch_dims, *,
   return lu_pivots_to_permutation_p.bind(
       x, permutation_size=permutation_size), 0
 
-def _lu_pivots_to_permutation_gpu_lowering(ctx, pivots, *, permutation_size):
-  if cuda_linalg:
-    return [cuda_linalg.lu_pivots_to_permutation_mhlo(
-        pivots, permutation_size=permutation_size)]
-  if hip_linalg:
-    return [hip_linalg.lu_pivots_to_permutation_mhlo(
-        pivots, permutation_size=permutation_size)]
+def _lu_pivots_to_permutation_gpu_lowering(lowering, ctx, pivots, *,
+                                           permutation_size):
+  return [lowering(pivots, permutation_size=permutation_size)]
 
 
 lu_pivots_to_permutation_p = Primitive('lu_pivots_to_permutation')
@@ -874,15 +972,32 @@ mlir.register_lowering(
     lu_pivots_to_permutation_p,
     mlir.lower_fun(_generic_lu_pivots_to_permutation, multiple_results=False))
 
+if gpu_linalg:
+  mlir.register_lowering(
+      lu_pivots_to_permutation_p,
+      partial(_lu_pivots_to_permutation_gpu_lowering,
+              gpu_linalg.cuda_lu_pivots_to_permutation),
+      platform='cuda')
+  mlir.register_lowering(
+      lu_pivots_to_permutation_p,
+      partial(_lu_pivots_to_permutation_gpu_lowering,
+              gpu_linalg.rocm_lu_pivots_to_permutation),
+      platform='rocm')
+
+
 if cuda_linalg:
   mlir.register_lowering(lu_pivots_to_permutation_p,
-                         _lu_pivots_to_permutation_gpu_lowering,
-                         platform='gpu')
+                         partial(_lu_pivots_to_permutation_gpu_lowering,
+                                 cuda_linalg.lu_pivots_to_permutation_mhlo),
+                         platform='cuda')
 
 if hip_linalg:
   mlir.register_lowering(lu_pivots_to_permutation_p,
-                         _lu_pivots_to_permutation_gpu_lowering,
-                         platform='gpu')
+                         partial(_lu_pivots_to_permutation_gpu_lowering,
+                                 hip_linalg.lu_pivots_to_permutation_mhlo),
+                         platform='rocm')
+
+
 # LU decomposition
 
 # Computes a pivoted LU decomposition such that
@@ -1077,6 +1192,14 @@ mlir.register_lowering(lu_p,
                        partial(_lu_cpu_gpu_lowering, lapack.getrf_mhlo),
                        platform='cpu')
 
+if gpu_solver is not None:
+  mlir.register_lowering(
+      lu_p, partial(_lu_cpu_gpu_lowering, gpu_solver.cuda_getrf),
+      platform='cuda')
+  mlir.register_lowering(
+      lu_p, partial(_lu_cpu_gpu_lowering, gpu_solver.rocm_getrf),
+      platform='rocm')
+
 if solver_apis is not None:
   mlir.register_lowering(
       lu_p, partial(_lu_cpu_gpu_lowering, solver_apis.getrf_mhlo),
@@ -1142,7 +1265,7 @@ def lu_solve(lu, permutation, b, trans=0):
 
 # QR decomposition
 
-def qr_impl(operand, full_matrices):
+def _qr_impl(operand, *, full_matrices):
   q, r = xla.apply_primitive(qr_p, operand, full_matrices=full_matrices)
   return q, r
 
@@ -1155,7 +1278,7 @@ def _qr_translation_rule(ctx, avals_in, avals_out, operand, *, full_matrices):
             _zeros_like_xla(ctx.builder, avals_out[1])]
   return xops.QR(operand, full_matrices)
 
-def qr_abstract_eval(operand, full_matrices):
+def _qr_abstract_eval(operand, *, full_matrices):
   if isinstance(operand, ShapedArray):
     if operand.ndim < 2:
       raise ValueError("Argument to QR decomposition must have ndims >= 2")
@@ -1168,7 +1291,7 @@ def qr_abstract_eval(operand, full_matrices):
     r = operand
   return q, r
 
-def qr_jvp_rule(primals, tangents, full_matrices):
+def qr_jvp_rule(primals, tangents, *, full_matrices):
   # See j-towns.github.io/papers/qr-derivative.pdf for a terse derivation.
   x, = primals
   dx, = tangents
@@ -1188,7 +1311,7 @@ def qr_jvp_rule(primals, tangents, full_matrices):
   dr = jnp.matmul(qt_dx_rinv - do, r)
   return (q, r), (dq, dr)
 
-def qr_batching_rule(batched_args, batch_dims, full_matrices):
+def _qr_batching_rule(batched_args, batch_dims, *, full_matrices):
   x, = batched_args
   bd, = batch_dims
   x = batching.moveaxis(x, bd, 0)
@@ -1227,11 +1350,18 @@ def _qr_cpu_gpu_lowering(geqrf_impl, orgqr_impl, ctx, operand, *,
                      mlir.dense_int_elements(list(batch_dims) + [n, n]),
                      mlir.dense_int_elements([1] * len(dims))).result
   else:
-    q = mhlo.PadOp(mlir.aval_to_ir_type(q_aval), r,
-                   mlir.ir_constant(np.array(0, dtype=operand_aval.dtype)),
-                   mlir.dense_int_elements([0] * len(dims)),
-                   mlir.dense_int_elements([0] * (len(dims) - 1) + [m - n]),
-                   mlir.dense_int_elements([0] * len(dims))).result
+    if jax._src.lib.mlir_api_version < 15:
+      q = mhlo.PadOp(mlir.aval_to_ir_type(q_aval), r,
+                     mlir.ir_constant(np.array(0, dtype=operand_aval.dtype)),
+                     mlir.dense_int_elements([0] * len(dims)),
+                     mlir.dense_int_elements([0] * (len(dims) - 1) + [m - n]),
+                     mlir.dense_int_elements([0] * len(dims))).result
+    else:
+      q = mhlo.PadOp(r,
+                     mlir.ir_constant(np.array(0, dtype=operand_aval.dtype)),
+                     mlir.dense_int_elements([0] * len(dims)),
+                     mlir.dense_int_elements([0] * (len(dims) - 1) + [m - n]),
+                     mlir.dense_int_elements([0] * len(dims))).result
     q, info_orgqr = orgqr_impl(operand_aval.dtype, q, tau)
   if info_geqrf is not None:
     zeros = mlir.full_like_aval(0, ShapedArray(batch_dims, np.dtype(np.int32)))
@@ -1258,16 +1388,29 @@ def _qr_cpu_gpu_lowering(geqrf_impl, orgqr_impl, ctx, operand, *,
 
 qr_p = Primitive('qr')
 qr_p.multiple_results = True
-qr_p.def_impl(qr_impl)
-qr_p.def_abstract_eval(qr_abstract_eval)
+qr_p.def_impl(_qr_impl)
+qr_p.def_abstract_eval(_qr_abstract_eval)
 xla.register_translation(qr_p, _qr_translation_rule)
 ad.primitive_jvps[qr_p] = qr_jvp_rule
-batching.primitive_batchers[qr_p] = qr_batching_rule
+batching.primitive_batchers[qr_p] = _qr_batching_rule
 
 mlir.register_lowering(
     qr_p, partial(_qr_cpu_gpu_lowering, lapack.geqrf_mhlo, lapack.orgqr_mhlo),
     platform='cpu')
 
+if gpu_solver is not None:
+  mlir.register_lowering(
+      qr_p,
+      partial(_qr_cpu_gpu_lowering, gpu_solver.cuda_geqrf,
+              gpu_solver.cuda_orgqr),
+      platform='cuda')
+  mlir.register_lowering(
+      qr_p,
+      partial(_qr_cpu_gpu_lowering, gpu_solver.rocm_geqrf,
+              gpu_solver.rocm_orgqr),
+      platform='rocm')
+
+
 if solver_apis is not None:
   mlir.register_lowering(
       qr_p,
@@ -1276,7 +1419,7 @@ if solver_apis is not None:
 
 # Singular value decomposition
 
-def svd_impl(operand, full_matrices, compute_uv):
+def _svd_impl(operand, *, full_matrices, compute_uv):
   return xla.apply_primitive(svd_p, operand, full_matrices=full_matrices,
                              compute_uv=compute_uv)
 
@@ -1292,7 +1435,7 @@ def _eye_like_xla(c, aval):
               xops.Iota(c, iota_shape, len(aval.shape) - 2))
   return xops.ConvertElementType(x, xla.dtype_to_primitive_type(aval.dtype))
 
-def svd_abstract_eval(operand, full_matrices, compute_uv):
+def _svd_abstract_eval(operand, *, full_matrices, compute_uv):
   if isinstance(operand, ShapedArray):
     if operand.ndim < 2:
       raise ValueError("Argument to singular value decomposition must have ndims >= 2")
@@ -1311,7 +1454,7 @@ def svd_abstract_eval(operand, full_matrices, compute_uv):
   else:
     raise NotImplementedError
 
-def svd_jvp_rule(primals, tangents, full_matrices, compute_uv):
+def _svd_jvp_rule(primals, tangents, *, full_matrices, compute_uv):
   A, = primals
   dA, = tangents
   s, U, Vt = svd_p.bind(A, full_matrices=False, compute_uv=True)
@@ -1436,7 +1579,7 @@ def _svd_tpu_lowering_rule(ctx, operand, *, full_matrices, compute_uv):
   return mlir.lower_fun(_svd_tpu, multiple_results=True)(
       ctx, operand, full_matrices=full_matrices, compute_uv=compute_uv)
 
-def svd_batching_rule(batched_args, batch_dims, full_matrices, compute_uv):
+def _svd_batching_rule(batched_args, batch_dims, *, full_matrices, compute_uv):
   x, = batched_args
   bd, = batch_dims
   x = batching.moveaxis(x, bd, 0)
@@ -1449,15 +1592,23 @@ def svd_batching_rule(batched_args, batch_dims, full_matrices, compute_uv):
 
 svd_p = Primitive('svd')
 svd_p.multiple_results = True
-svd_p.def_impl(svd_impl)
-svd_p.def_abstract_eval(svd_abstract_eval)
-ad.primitive_jvps[svd_p] = svd_jvp_rule
-batching.primitive_batchers[svd_p] = svd_batching_rule
+svd_p.def_impl(_svd_impl)
+svd_p.def_abstract_eval(_svd_abstract_eval)
+ad.primitive_jvps[svd_p] = _svd_jvp_rule
+batching.primitive_batchers[svd_p] = _svd_batching_rule
 
 mlir.register_lowering(
     svd_p, partial(_svd_cpu_gpu_lowering, lapack.gesdd_mhlo),
     platform='cpu')
 
+if gpu_solver is not None:
+  mlir.register_lowering(
+    svd_p, partial(_svd_cpu_gpu_lowering, gpu_solver.cuda_gesvd),
+    platform='cuda')
+  mlir.register_lowering(
+    svd_p, partial(_svd_cpu_gpu_lowering, gpu_solver.rocm_gesvd),
+    platform='rocm')
+
 if solver_apis is not None:
   mlir.register_lowering(
     svd_p, partial(_svd_cpu_gpu_lowering, solver_apis.gesvd_mhlo),
@@ -1465,8 +1616,8 @@ if solver_apis is not None:
 
 mlir.register_lowering(svd_p, _svd_tpu_lowering_rule)
 
-def _tridiagonal_solve_gpu_lowering(ctx, dl, d, du, b, *, m, n, ldb, t):
-  return [sparse_apis.gtsv2_mhlo(dl, d, du, b, m=m, n=n, ldb=ldb, t=t)]
+def _tridiagonal_solve_gpu_lowering(lowering, ctx, dl, d, du, b, *, m, n, ldb, t):
+  return [lowering(dl, d, du, b, m=m, n=n, ldb=ldb, t=t)]
 
 tridiagonal_solve_p = Primitive('tridiagonal_solve')
 tridiagonal_solve_p.multiple_results = False
@@ -1479,6 +1630,17 @@ if sparse_apis and hasattr(sparse_apis, "gtsv2"):
                          _tridiagonal_solve_gpu_lowering,
                          platform='gpu')
 
+if gpu_sparse:
+  mlir.register_lowering(
+      tridiagonal_solve_p,
+      partial(_tridiagonal_solve_gpu_lowering, gpu_sparse.cuda_gtsv2),
+      platform='cuda')
+  mlir.register_lowering(
+      tridiagonal_solve_p,
+      partial(_tridiagonal_solve_gpu_lowering, gpu_sparse.rocm_gtsv2),
+      platform='rocm')
+
+
 def _tridiagonal_solve_jax(dl, d, du, b, **kw):
   """Pure JAX implementation of `tridiagonal_solve`."""
   prepend_zero = lambda x: jnp.append(jnp.zeros([1], dtype=x.dtype), x[:-1])
@@ -1562,7 +1724,8 @@ def tridiagonal_solve(dl, d, du, b):
 # Schur Decomposition
 
 
-def schur(x,
+@_warn_on_positional_kwargs
+def schur(x, *,
           compute_schur_vectors=True,
           sort_eig_vals=False,
           select_callable=None):
