@@ -22,7 +22,8 @@ import warnings
 from typing import Optional, Tuple
 
 __all__ = [
-  'cuda_linalg', 'cuda_prng', 'cusolver', 'hip_linalg', 'hip_prng',
+  'cuda_linalg', 'cuda_prng', 'cusolver', 'gpu_linalg', 'gpu_prng',
+  'gpu_sparse', 'hip_linalg', 'hip_prng',
   'hipsolver','jaxlib', 'lapack', 'pocketfft', 'pytree',
    'tpu_driver_client', 'version', 'xla_client', 'xla_extension',
 ]
@@ -120,6 +121,9 @@ pytree = xla_client._xla.pytree
 jax_jit = xla_client._xla.jax_jit
 pmap_lib = xla_client._xla.pmap_lib
 
+# TODO(phawkins): make gpu_... unconditional after jaxlib >= 0.3.11
+# becomes the minimum; remove cuda_... and hip_....
+
 try:
   import jaxlib.cusolver as cusolver  # pytype: disable=import-error
 except ImportError:
@@ -130,6 +134,11 @@ try:
 except ImportError:
   hipsolver = None
 
+try:
+  import jaxlib.gpu_solver as gpu_solver  # pytype: disable=import-error
+except ImportError:
+  gpu_solver = None
+
 try:
   import jaxlib.cusparse as cusparse  # pytype: disable=import-error
 except ImportError:
@@ -140,6 +149,11 @@ try:
 except ImportError:
   hipsparse = None
 
+try:
+  import jaxlib.gpu_sparse as gpu_sparse  # pytype: disable=import-error
+except ImportError:
+  gpu_sparse = None
+
 sparse_apis = cusparse or hipsparse or None
 solver_apis = cusolver or hipsolver or None
 
@@ -153,6 +167,11 @@ try:
 except ImportError:
   hip_prng = None
 
+try:
+  import jaxlib.gpu_prng as gpu_prng  # pytype: disable=import-error
+except ImportError:
+  gpu_prng = None
+
 try:
   import jaxlib.cuda_linalg as cuda_linalg  # pytype: disable=import-error
 except ImportError:
@@ -163,6 +182,11 @@ try:
 except ImportError:
   hip_linalg = None
 
+try:
+  import jaxlib.gpu_linalg as gpu_linalg  # pytype: disable=import-error
+except ImportError:
+  gpu_linalg = None
+
 # Jaxlib code is split between the Jax and the Tensorflow repositories.
 # Only for the internal usage of the JAX developers, we expose a version
 # number that can be used to perform changes without breaking the main
