@@ -16,9 +16,10 @@
 import jaxlib.mlir.dialects.builtin as builtin
 import jaxlib.mlir.dialects.chlo as chlo
 import jaxlib.mlir.dialects.mhlo as mhlo
+import jaxlib.mlir.dialects.func as func
 try:
-  import jaxlib.mlir.dialects.func as func  # pytype: disable=import-error
-except ImportError:
-  # TODO(phawkins): remove std dialect after MLIR change lands, most likely in
-  # jaxlib > 0.3.0.
-  import jaxlib.mlir.dialects.std as func  # pytype: disable=import-error
+  import jaxlib.mlir.dialects.sparse_tensor as sparse_tensor
+except (ModuleNotFoundError, ImportError):
+  # TODO(ajcbik,phawkins): make this unconditional when jaxlib > 0.3.7
+  #                        is the minimum version.
+  pass
