@@ -37,6 +37,9 @@ import jax._src.lib.xla_bridge
 
 import numpy as np
 import tensorflow as tf  # type: ignore[import]
+# pylint: disable=g-direct-tensorflow-import
+from tensorflow.compiler.tf2xla.python import xla as tfxla  # type: ignore[import]
+# pylint: enable=g-direct-tensorflow-import
 
 config.parse_flags_with_absl()
 
@@ -637,6 +640,26 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     self.assertLen(jax.tree_leaves(m.b), 2)
     self.assertLen(jax.tree_leaves(m.c), 2)
 
+  def test_issue_10586(self):
+
+    class JaxModule(tf.Module):
+      def __init__(self):
+        self._params = {'w': tf.Variable(tf.ones([784, 10]), name='w'),
+                        'b': tf.Variable(tf.ones([10]), name='b')}
+
+      def __call__(self, x):
+        return jax2tf.convert(lambda p, x: x @ p['w'] + p['b'])(self._params, x)
+
+    net = JaxModule()
+    images = tf.ones([1, 784])
+
+    with tf.GradientTape() as tape:
+      loss = tf.reduce_sum(net(images))
+    params = tape.watched_variables()
+    grads = tape.gradient(loss, params)
+    for var, grad in zip(params, grads):
+      self.assertEqual(var.shape, grad.shape, msg=var.name)
+
   def test_custom_jvp(self):
     """Conversion of function with custom JVP"""
 
@@ -1144,4 +1167,7 @@ class Jax2TfTest(tf_test_util.JaxToTfTestCase):
     )
 
 if __name__ == "__main__":
+  # TODO: Remove once tensorflow is 2.10.0 everywhere.
+  if not hasattr(tfxla, 'optimization_barrier'):
+    jax.config.update('jax_remat_opt_barrier', False)
   absltest.main(testLoader=jtu.JaxTestLoader())
