@@ -1057,12 +1057,12 @@ def _dynamic_jaxpr_process_xmap(self, primitive, f, tracers, params):
 pe.DynamicJaxprTrace.process_xmap = _dynamic_jaxpr_process_xmap  # type: ignore
 
 def _xmap_partial_eval_custom_params_updater(
-    unks_in: Sequence[bool],
+    unks_in: Sequence[bool], inst_in: Sequence[bool],
     kept_outs_known: Sequence[bool], kept_outs_staged: Sequence[bool],
     num_res: int, params_known: dict, params_staged: dict
   ) -> Tuple[dict, dict]:
-  assert params_known['spmd_in_axes'] is None and params_known['spmd_out_axes'] is None
-  assert params_staged['spmd_in_axes'] is None and params_staged['spmd_out_axes'] is None
+  assert params_known['spmd_in_axes'] is None is params_known['spmd_out_axes']
+  assert params_staged['spmd_in_axes'] is None is params_staged['spmd_out_axes']
 
   # pruned inputs to jaxpr_known according to unks_in
   donated_invars_known, _ = pe.partition_list(unks_in, params_known['donated_invars'])
@@ -1085,13 +1085,15 @@ def _xmap_partial_eval_custom_params_updater(
   assert len(new_params_known['in_axes']) == len(params_known['call_jaxpr'].invars)
   assert len(new_params_known['out_axes']) == len(params_known['call_jaxpr'].outvars)
 
-  # added num_res new inputs to jaxpr_staged
-  donated_invars_staged = (*(False for _ in range(num_res)), *params_staged['donated_invars'])
+  # added num_res new inputs to jaxpr_staged, and pruning according to inst_in
+  _, donated_invars_staged = pe.partition_list(inst_in, params_staged['donated_invars'])
+  donated_invars_staged = [False] * num_res + donated_invars_staged
+  _, in_axes_staged = pe.partition_list(inst_in, params_staged['in_axes'])
+  in_axes_staged = [*residual_axes, *in_axes_staged]
   _, out_axes_staged = pe.partition_list(kept_outs_staged, params_staged['out_axes'])
-  new_params_staged = dict(params_staged,
-                           in_axes=(*residual_axes, *params_staged['in_axes']),
+  new_params_staged = dict(params_staged, in_axes=tuple(in_axes_staged),
                            out_axes=tuple(out_axes_staged),
-                           donated_invars=donated_invars_staged)
+                           donated_invars=tuple(donated_invars_staged))
   assert len(new_params_staged['in_axes']) == len(params_staged['call_jaxpr'].invars)
   assert len(new_params_staged['out_axes']) == len(params_staged['call_jaxpr'].outvars)
   return new_params_known, new_params_staged
