@@ -24,7 +24,8 @@ Batched-coordinate (BCOO) sparse matrices
 -----------------------------------------
 The main high-level sparse object currently available in JAX is the :class:`BCOO`,
 or *batched coordinate* sparse array, which offers a compressed storage format compatible
-with JAX functions.
+with JAX transformations, in particular JIT (e.g. :func:`jax.jit`), batching
+(e.g. :func:`jax.vmap`) and autodiff (e.g. :func:`jax.grad`).
 
 Here is an example of creating a sparse array from a dense array:
 
@@ -187,7 +188,7 @@ from jax.experimental.sparse.ad import (
     value_and_grad as value_and_grad,
 )
 from jax.experimental.sparse.bcoo import (
-    bcoo_add_batch_dim as bcoo_add_batch_dim,
+    bcoo_add_batch_dim,  # TODO(jakevdp) remove this function
     bcoo_broadcast_in_dim as bcoo_broadcast_in_dim,
     bcoo_concatenate as bcoo_concatenate,
     bcoo_dot_general as bcoo_dot_general,
@@ -200,6 +201,7 @@ from jax.experimental.sparse.bcoo import (
     bcoo_fromdense_p as bcoo_fromdense_p,
     bcoo_multiply_dense as bcoo_multiply_dense,
     bcoo_multiply_sparse as bcoo_multiply_sparse,
+    bcoo_update_layout as bcoo_update_layout,
     bcoo_reduce_sum as bcoo_reduce_sum,
     bcoo_reshape as bcoo_reshape,
     bcoo_sort_indices as bcoo_sort_indices,
@@ -222,6 +224,8 @@ from jax.experimental.sparse.api import (
 
 from jax.experimental.sparse.util import (
     CuSparseEfficiencyWarning as CuSparseEfficiencyWarning,
+    SparseEfficiencyError as SparseEfficiencyError,
+    SparseEfficiencyWarning as SparseEfficiencyWarning,
 )
 
 from jax.experimental.sparse.coo import (
