@@ -148,10 +148,6 @@ class SparsifyEnv:
       raise RuntimeError("Internal: requested indices from spvalue with indices_ref=None")
     return self._buffers[spvalue.indices_ref]
 
-  def unit(self):
-    """Add a new unit representation to the sparsify environment."""
-    return SparsifyValue((), None, None)
-
   def dense(self, data):
     """Add a new dense array to the sparsify environment."""
     return SparsifyValue(np.shape(data), self._push(data), None)
@@ -186,9 +182,6 @@ class SparsifyValue(NamedTuple):
   def is_sparse(self):
     return self.indices_ref is not None
 
-  def is_unit(self):
-    return self.data_ref is None
-
 
 _is_bcoo = lambda arg: isinstance(arg, BCOO)
 _is_spvalue = lambda arg: isinstance(arg, SparsifyValue)
@@ -202,8 +195,6 @@ def arrays_to_spvalues(
   def array_to_spvalue(arg):
     if isinstance(arg, BCOO):
       return spenv.sparse(arg.shape, arg.data, arg.indices)
-    elif core.get_aval(arg) is core.abstract_unit:
-      return spenv.unit()
     else:
       return spenv.dense(arg)
   return tree_map(array_to_spvalue, args, is_leaf=_is_bcoo)
@@ -218,8 +209,6 @@ def spvalues_to_arrays(
     if spvalue.is_sparse():
       assert spvalue.indices_ref is not None
       return BCOO((spenv.data(spvalue), spenv.indices(spvalue)), shape=spvalue.shape)
-    elif spvalue.is_unit():
-      return core.unit
     else:
       return spenv.data(spvalue)
   return tree_map(spvalue_to_array, spvalues, is_leaf=_is_spvalue)
@@ -231,11 +220,8 @@ def spvalues_to_avals(
     ) -> Any:
   """Convert a pytree of spvalues to an equivalent pytree of abstract values."""
   def spvalue_to_aval(spvalue):
-    if spvalue.is_unit():
-      return core.abstract_unit
-    else:
-      data = spenv.data(spvalue)
-      return core.ShapedArray(spvalue.shape, data.dtype, data.aval.weak_type)
+    data = spenv.data(spvalue)
+    return core.ShapedArray(spvalue.shape, data.dtype, data.aval.weak_type)
   return tree_map(spvalue_to_aval, spvalues, is_leaf=_is_spvalue)
 
 
@@ -372,8 +358,6 @@ def eval_sparse(
     assert a is not None
     env[var] = a
 
-  # TODO: handle unitvar at all?
-  #write_buffer(core.unitvar, core.unit)
   safe_map(write_buffer, jaxpr.constvars, consts)
   safe_map(write, jaxpr.invars, spvalues)
 
@@ -613,6 +597,13 @@ def _squeeze_sparse(spenv, *spvalues, dimensions):
 
 sparse_rules[lax.squeeze_p] = _squeeze_sparse
 
+def _reshape_sparse(spenv, *spvalues, new_sizes, dimensions):
+  operand, = spvalues_to_arrays(spenv, spvalues)
+  result = sparse.bcoo_reshape(operand, new_sizes=new_sizes, dimensions=dimensions)
+  return arrays_to_spvalues(spenv, (result,))
+
+sparse_rules[lax.reshape_p] = _reshape_sparse
+
 def _sparsify_jaxpr(spenv, jaxpr, *spvalues):
   # TODO(jakevdp): currently this approach discards all information about
   #   shared data & indices when generating the sparsified jaxpr. The
@@ -736,6 +727,10 @@ def _sum(self, *args, **kwargs):
   """Sum array along axis."""
   return sparsify(lambda x: x.sum(*args, **kwargs))(self)
 
+def _reshape(self, *args, **kwargs):
+  """Sum array along axis."""
+  return sparsify(lambda x: x.reshape(*args, **kwargs))(self)
+
 def _sparse_rewriting_take(arr, idx, indices_are_sorted=False, unique_indices=False,
                            mode=None, fill_value=None):
   # mirrors lax_numpy._rewriting_take.
@@ -751,6 +746,7 @@ def _sparse_rewriting_take(arr, idx, indices_are_sorted=False, unique_indices=Fa
 _swap_args = lambda f: lambda a, b: f(b, a)
 
 _bcoo_methods = {
+  'reshape': _reshape,
   'sum': _sum,
   "__neg__": sparsify(jnp.negative),
   "__pos__": sparsify(jnp.positive),
