@@ -79,11 +79,13 @@ def i64_attr(i): return ir.IntegerAttr.get(ir.IntegerType.get_signless(64), i)
 
 def shape_tensor(sizes: Sequence[Union[int, ir.RankedTensorType]]
                  ) -> ir.RankedTensorType:
-  sizes = [ir_constant(np.array(d, np.dtype('int32'))) if type(d) is int else d
-           for d in sizes]
   int1d = aval_to_ir_type(core.ShapedArray((1,), np.dtype('int32')))
-  return mhlo.ConcatenateOp([mhlo.ReshapeOp(int1d, d) for d in sizes],
-                            i64_attr(0)).results
+  d, *ds = [ir_constant(np.array([d], np.dtype('int32'))) if type(d) is int
+            else mhlo.ReshapeOp(int1d, d) for d in sizes]
+  if not ds:
+    return d
+  else:
+    return mhlo.ConcatenateOp([d, *ds], i64_attr(0)).result
 
 
 # IR Types
@@ -438,9 +440,15 @@ _platform_specific_lowerings = collections.defaultdict(dict)
 
 def register_lowering(prim: core.Primitive, rule: LoweringRule,
                       platform: Optional[str] = None):
-  ts = (_lowerings if platform is None
-        else _platform_specific_lowerings[platform])
-  ts[prim] = rule
+  if platform is None:
+    _lowerings[prim] = rule
+  else:
+    # For backward compatibility reasons, we allow rules to be registered
+    # under "gpu" even though the platforms are now called "cuda" and "rocm".
+    # TODO(phawkins): fix up users to specify either "cuda" or "rocm" and remove
+    # this expansion.
+    for p in xb.expand_platform_alias(platform):
+      _platform_specific_lowerings[p][prim] = rule
 
 
 def _unwrap_singleton_ir_values(x): return x[0] if len(x) == 1 else x
@@ -496,7 +504,9 @@ def lower_jaxpr_to_module(
   """Lowers a top-level jaxpr to an MHLO module.
 
   Handles the quirks of the argument/return value passing conventions of the
-  runtime."""
+  runtime.
+  """
+  platform = xb.canonicalize_platform(platform)
   if not xb.is_known_platform(platform):
     raise ValueError(f"Unknown platform {platform}")
   input_output_aliases = None
@@ -512,7 +522,7 @@ def lower_jaxpr_to_module(
         sharded_aval(out_aval, out_sharding)
         for out_aval, out_sharding in zip(out_avals, result_shardings)
     ]
-  platforms_with_donation = ("gpu", "tpu")
+  platforms_with_donation = ("cuda", "rocm", "tpu")
   if platform in platforms_with_donation:
     input_output_aliases, donated_args = _set_up_aliases(
         in_avals, out_avals, donated_args)
@@ -859,6 +869,7 @@ def jaxpr_subcomp(ctx: ModuleContext, jaxpr: core.Jaxpr,
 
   Assumes that an MLIR context, location, and insertion point are set.
   """
+  assert ctx.platform != "gpu"
   def read(v: core.Var) -> Sequence[ir.Value]:
     if type(v) is core.Literal:
       return ir_constants(v.val, canonicalize_types=True)
@@ -975,13 +986,13 @@ def lower_fun(fun: Callable, multiple_results: bool = True) -> Callable:
 
 def _call_lowering(fn_name, stack_name, call_jaxpr, backend, ctx, avals_in,
                    avals_out, tokens_in, *args):
+  if isinstance(call_jaxpr, core.Jaxpr):
+    call_jaxpr = core.ClosedJaxpr(call_jaxpr, ())
   xla.check_backend_matches(backend, ctx.platform)
   output_types = map(aval_to_ir_types, avals_out)
   flat_output_types = util.flatten(output_types)
   effects = tokens_in.effects()
-  symbol_name = lower_jaxpr_to_fun(ctx, fn_name,
-                                   core.ClosedJaxpr(call_jaxpr, ()),
-                                   effects).name.value
+  symbol_name = lower_jaxpr_to_fun(ctx, fn_name, call_jaxpr, effects).name.value
   args = [*tokens_in.tokens(), *args]
   call = func_dialect.CallOp(flat_output_types,
                              ir.FlatSymbolRefAttr.get(symbol_name),
@@ -1013,6 +1024,8 @@ def _named_call_lowering(ctx, *args, name, backend=None,
 
 register_lowering(core.named_call_p, _named_call_lowering)
 register_lowering(core.call_p, partial(_named_call_lowering, name="core_call"))
+register_lowering(core.closed_call_p,
+                  partial(_named_call_lowering, name="core_closed_call"))
 
 
 def full_like_aval(value, aval: core.ShapedArray) -> ir.Value:
@@ -1038,10 +1051,18 @@ register_lowering(ad_util.add_jaxvals_p, add_jaxvals_lowering)
 register_lowering(ad_util.stop_gradient_p, lambda ctx, x: [x])
 
 
-def compare_mhlo(x, y, direction, type):
+def compare_mhlo(x, y, direction: str, comparison_type: Optional[str] = None):
   """Creates mhlo.CompareOp."""
+  if comparison_type is None:
+    elem_type = ir.RankedTensorType(x.type).element_type
+    if ir.IntegerType.isinstance(elem_type):
+      comparison_type = ("UNSIGNED" if ir.IntegerType.is_unsigned(elem_type)
+                         else "SIGNED")
+    else:
+      comparison_type = "FLOAT"
+
   return mhlo.CompareOp(x, y, mhlo.ComparisonDirectionAttr.get(direction),
-                        mhlo.ComparisonTypeAttr.get(type))
+                        mhlo.ComparisonTypeAttr.get(comparison_type))
 
 def _minmax_mhlo(op, cmp, x, y):
   """Min/max that compares complex values lexicographically as pairs."""
@@ -1271,7 +1292,3 @@ def emit_python_callback(platform, callback,
 # # CHLO doesn't have a legalization for bf16 (b/203774470)
 # lax.erf_p,
 # lax.erfc_p,
-
-# # Not present in cHLO or mHLO (b/203798239), although we could just emit the
-# # lowered pattern ourselves.
-# lax.top_k_p,
