@@ -42,6 +42,7 @@ from jax._src.util import (prod, new_name_stack, safe_zip, safe_map,
 from jax._src.util import extend_name_stack as extend_name_stack  # noqa: F401
 from jax._src.util import wrap_name as wrap_name  # noqa: F401
 
+from jax._src.lib import xla_bridge as xb
 from jax._src.lib import xla_client as xc
 from jax.interpreters import partial_eval as pe
 from jax.interpreters import ad
@@ -337,7 +338,10 @@ def xla_destructure(c, ans):
 def check_backend_matches(inner_backend, outer_backend):
   # For nested calls, the outermost call sets the backend for all inner calls;
   # it's an error if the inner call has a conflicting explicit backend spec.
-  if inner_backend and inner_backend != outer_backend:
+  if inner_backend is None:
+    return
+  if (inner_backend != outer_backend and
+      outer_backend not in xb.expand_platform_alias(inner_backend)):
     raise ValueError(
         f"Outer-jit backend specification {outer_backend} must match explicit "
         f"inner-jit backend specification {inner_backend}.")
@@ -429,16 +433,17 @@ ad.primitive_transposes[xla_call_p] = partial(ad.call_transpose, xla_call_p)
 
 
 def _xla_call_partial_eval_custom_params_updater(
-    unks_in: Sequence[bool],
+    unks_in: Sequence[bool], inst_in: Sequence[bool],
     kept_outs_known: Sequence[bool], kept_outs_staged: Sequence[bool],
     num_res: int, params_known: dict, params_staged: dict
   ) -> Tuple[dict, dict]:
   # pruned inputs to jaxpr_known according to unks_in, so prune donated_invars
-  donated_invars_known, _ = partition_list(unks_in, params_known['donated_invars'])
-  new_params_known = dict(params_known, donated_invars=tuple(donated_invars_known))
+  donated_known, _ = partition_list(unks_in, params_known['donated_invars'])
+  new_params_known = dict(params_known, donated_invars=tuple(donated_known))
   # added num_res new inputs to jaxpr_staged, so extend donated_invars
-  donated_invars_staged = [*([False] * num_res), *params_staged['donated_invars']]
-  new_params_staged = dict(params_staged, donated_invars=tuple(donated_invars_staged))
+  _, donated_staged_ = partition_list(inst_in, params_staged['donated_invars'])
+  donated_staged = [False] * num_res + donated_staged_
+  new_params_staged = dict(params_staged, donated_invars=tuple(donated_staged))
   return new_params_known, new_params_staged
 pe.partial_eval_jaxpr_custom_rules[xla_call_p] = \
     partial(pe.call_partial_eval_custom_rule, 'call_jaxpr',
@@ -491,9 +496,16 @@ def register_collective_primitive(prim: core.Primitive):
 
 def register_translation(prim: core.Primitive, rule: TranslationRule, *,
                          platform: Optional[str] = None) -> None:
-  ts = (_translations if platform is None
-        else _backend_specific_translations[platform])
-  ts[prim] = rule
+  if platform is None:
+    _translations[prim] = rule
+  else:
+    # For backward compatibility reasons, we allow rules to be registered
+    # under "gpu" even though the platforms are now called "cuda" and "rocm".
+    # TODO(phawkins): fix up users to specify either "cuda" or "rocm" and remove
+    # this expansion.
+    for p in xb.expand_platform_alias(platform):
+      _backend_specific_translations[p][prim] = rule
+
 
 # As a temporary backward compatibility measure, we use an adapter class to
 # convert from the old styles of translation rules to the newer ones.
@@ -506,7 +518,9 @@ class _TranslationRuleAdapter:
     self._wrap_fn = wrap_fn
 
   def __setitem__(self, key: core.Primitive, value: Callable):
-    self._translations[key] = self._wrap_fn(key, value)
+    wrapped = self._wrap_fn(key, value)
+    for translations in self._translations:
+      translations[key] = wrapped
 
 
 def _wrap_old_translation(prim: core.Primitive, f: Callable) -> TranslationRule:
@@ -524,12 +538,14 @@ def _wrap_old_translation(prim: core.Primitive, f: Callable) -> TranslationRule:
 
 
 translations : _TranslationRuleAdapter
-translations = _TranslationRuleAdapter(_translations, _wrap_old_translation)
+translations = _TranslationRuleAdapter([_translations], _wrap_old_translation)
 
 class _BackendSpecificTranslationsAdapter(defaultdict):
   def __missing__(self, key):
+    translation_tables = [_backend_specific_translations[p]
+                          for p in xb.expand_platform_alias(key)]
     ret = self[key] = _TranslationRuleAdapter(
-        _backend_specific_translations[key], _wrap_old_translation)
+        translation_tables, _wrap_old_translation)
     return ret
 
 backend_specific_translations: Dict[str, _TranslationRuleAdapter]
@@ -544,7 +560,3 @@ def lower_fun(fun: Callable, *, multiple_results: bool, backend=None,
                        "Add an MLIR (MHLO) lowering via jax.interpreters.mlir "
                        "instead.")
   return f
-
-
-ad.primitive_transposes[core.named_call_p] = partial(ad.call_transpose,
-                                                     core.named_call_p)
